from ....tags.body.core import Body as BodyParentClass


class Body(BodyParentClass):
    def __init__(self, senior):
        super().__init__(senior)
        self._top = self.create.node.group()
        self._upper = self.create.node.group()
        self._middle = self.create.node.group()
        self._lower = self.create.node.group()
        self._footer = self.create.node.group()
        self._bottom = self.create.node.group()
        self._libraries = self.create.node.group()

    @property
    def top(self):
        return self._top

    @property
    def upper(self):
        return self._upper

    @property
    def middle(self):
        return self._middle

    @property
    def lower(self):
        return self._lower

    @property
    def footer(self):
        return self._footer

    @property
    def bottom(self):
        return self._bottom

    @property
    def libraries(self):
        return self._libraries



from ....tags.tbody.core import Tbody as TbodyParentClass
from .tr.core import Tr


class Tbody(TbodyParentClass):
    def __init__(self, senior):
        super().__init__(senior)
        self._rows = []
        self._row = None

        self._style = None

    @property
    def rows(self) -> list:
        return self._rows

    @property
    def row(self) -> Tr:
        if self._row is None:
            self.add_row()
        return self._row

    @property
    def cells(self): return self.row.cells

    @property
    def cell(self): return self.row.cell

    def add_cell(self, text=None): return self.row.add_cell(text)

    def add_row(self, data: list = None) -> Tr:
        self._row = Tr(self)
        self._rows.append(self._row)
        if data:
            for d in data:
                self.row.add_cell(d)
        return self._row

    @property
    def color(self):
        if self._style is None:
            from ..__templates__.color import Color
            self._style = Color(self)
        return self._style



from ...nodes import Tag


class A(Tag):
    output_break_inner = False

    def __init__(self, senior, text=None, url: str = None):
        super().__init__(senior, 'a')
        self.add_junior(text)
        self.attributes.set("href", url if url else '#')



class Create:
    def __init__(self, sir):
        self.senior = sir
        self._nodes = None
        self._tags = None
        self._components = None

    @property
    def node(self):
        if self._nodes is None:
            from .nodes.core import Nodes
            self._nodes = Nodes(self.senior)
        return self._nodes

    @property
    def tag(self):
        if self._tags is None:
            from .tags.core import Tags
            self._tags = Tags(self.senior)
        return self._tags

    @property
    def component(self):
        if self._components is None:
            from .components.core import Components
            self._components = Components(self.senior)
        return self._components



class Nodes:
    def __init__(self, sir):
        self.senior = sir

    def code(self, text: str = None):
        from .....nodes import Code
        return Code(self.senior, text)

    def content(self, text: str = None):
        from .....nodes import Content
        return Content(self.senior, text)

    def comment(self, text: str = None):
        from .....nodes import Comment
        return Comment(self.senior, text)

    def group(self):
        from .....nodes import Group
        return Group(self.senior)

    def tag(self, name: str):
        from .....nodes import Tag
        return Tag(self.senior, name)



from ..node.core import Node


class Comment(Node):
    def __init__(self, sir, text: str = None):
        super().__init__(sir)
        self.text = text if text is not None else ''

    def output(self):
        s = ""
        if self.document.output_break:
            s += "\n" + self.document.output_retraction * self.level
        s += f"<!--[{self.text.replace("&", "&amp;").replace("<", "&lt;").replace(">", "&gt;")}]-->"
        self.document.output_next_breakable = True
        return s



from ...tags import Html as HtmlParentClass
from .head.core import Head
from .body.core import Body


class Html(HtmlParentClass):
    def __init__(self, senior, title: str = 'Untitled', lang: str = None):
        super().__init__(senior, lang)
        self._head = Head(self)
        self._body = Body(self)
        self._library_files = []
        self.title = title
        self.add_meta().set_attribute_charset('utf-8')
        css = self.add_head_css_link('https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/css/bootstrap.min.css')
        css.attributes.set('rel', 'stylesheet')
        css.attributes.set('integrity', 'sha384-QWTKZyjpPEjISv5WaRU9OFeRpok6YctnYmDr5pNlyT2bRjXh0JMhjY6hW+ALEwIH')
        css.attributes.set('crossorigin', 'anonymous')
        css = self.add_head_css_link('https://cdn.jsdelivr.net/gh/hebill/hebill_lib/hebill.css')
        # css = self.add_head_css_link('https://cdn.jsdelivr.net/gh/hebill/hebill_lib/hebill.css')
        css.attributes.set('rel', 'stylesheet')
        # css.attributes.set('integrity', 'sha384-QWTKZyjpPEjISv5WaRU9OFeRpok6YctnYmDr5pNlyT2bRjXh0JMhjY6hW+ALEwIH')
        # https://cdn.jsdelivr.net/gh/hebill/hebill_lib@releases/tag/1.0.0
        css.attributes.set('crossorigin', 'anonymous')
        js = self.add_library_js_link_to_body(
            'https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/js/bootstrap.bundle.min.js')
        js.attributes.set('integrity', 'sha384-YvpcrYf0tY3lHB60NNkmXc5s9fDVZLESaAA55NDzOxhy9GkcIdslK1eN7N6jIeHz')
        js.attributes.set('crossorigin', 'anonymous')

    @property
    def head(self):
        return self._head

    @property
    def title(self):
        return self.head.title

    @title.setter
    def title(self, title):
        self.head.title = title

    @property
    def body(self):
        return self._body

    ##################################################
    @property
    def library_files(self):
        return self._library_files

    def is_library_file_added(self, url):
        return url in self._library_files

    def add_library_js_link_to_head(self, url):
        if self.is_library_file_added(url):
            return True
        self._library_files.append(url)
        return self.head.libraries.create.tag.script(url)

    def add_library_js_link_to_body(self, url):
        if self.is_library_file_added(url):
            return True
        self._library_files.append(url)
        return self.body.libraries.create.tag.script(url)

    def add_head_css_link(self, url):
        if self.is_library_file_added(url):
            return True
        self._library_files.append(url)
        return self.head.libraries.create.tag.link(url)

    def add_meta(self):
        return self.head.libraries.create.tag.meta()



from ...nodes import Tag


class Html(Tag):
    def __init__(self, senior, lang: str = None):
        super().__init__(senior, 'html')
        if lang is not None:
            self.attributes["lang"] = lang

    def set_attribute_lang(self, lang: str):
        self.attributes["lang"] = lang



from .....tags.tr.core import Tr as TrParentClass
from .td.core import Td


class Tr(TrParentClass):
    def __init__(self, senior):
        super().__init__(senior)
        self._cells = []
        self._cell = None

        self._style = None

    @property
    def cells(self) -> list:
        return self._cells

    @property
    def cell(self) -> Td:
        if self._cell is None:
            self.add_cell()
        return self._cell

    def add_cell(self, text=None) -> Td:
        self._cell = Td(self, text)
        self._cells.append(self._cell)
        return self._cell

    @property
    def color(self):
        if self._style is None:
            from ....table.__templates__.color import Color
            self._style = Color(self)
        return self._style

    def set_active(self):
        self.attributes.classes.append('table-active')



from colorama import Fore, Back


class Color:
    def __init__(self, terminal, fore=False):
        from .core import Terminal
        self._terminal: Terminal = terminal
        self._fore = fore
        self._FB = Fore if fore else Back

    def _set(self, color):
        if self._fore:
            self._terminal.set_color(color)
        else:
            self._terminal.set_background(color)
        return self._terminal

    @property
    def black(self): return self._set(self._FB.BLACK)

    @property
    def blue(self): return self._set(self._FB.BLUE)

    @property
    def cyan(self): return self._set(self._FB.BLUE)

    @property
    def green(self): return self._set(self._FB.GREEN)

    @property
    def magenta(self): return self._set(self._FB.MAGENTA)

    @property
    def red(self): return self._set(self._FB.RED)

    @property
    def yellow(self): return self._set(self._FB.YELLOW)

    @property
    def white(self): return self._set(self._FB.WHITE)

    @property
    def light_black(self): return self._set(self._FB.LIGHTBLACK_EX)

    @property
    def light_blue(self): return self._set(self._FB.LIGHTBLUE_EX)

    @property
    def light_cyan(self): return self._set(self._FB.LIGHTCYAN_EX)

    @property
    def light_green(self): return self._set(self._FB.LIGHTGREEN_EX)

    @property
    def light_magenta(self): return self._set(self._FB.LIGHTMAGENTA_EX)

    @property
    def light_red(self): return self._set(self._FB.LIGHTRED_EX)

    @property
    def light_yellow(self): return self._set(self._FB.LIGHTYELLOW_EX)



from ...tags.div.core import Div


class Container(Div):
    def __init__(self, senior):
        super().__init__(senior)
        self.attributes.classes.set("container")
        self._size = None

    @property
    def size(self):
        if self._size is None:
            from .size import Size
            self._size = Size(self)
        return self._size



class Styles(dict):
    def reset(self, styles: str | list = None) -> bool:
        self.clear()
        if styles is None:
            return True
        return self.sets(styles)

    def set(self, name: str, style: str):
        self[name.strip()] = style.strip()

    def sets(self, styles: dict | str = None) -> bool:
        if isinstance(styles, str):
            if styles.strip() == '':
                return True
            if ';' not in styles:
                styles += ';'
            x = styles.split(';')
            for style in x:
                if ':' in style:
                    n, v = style.split(':')
                    self.set(n, v)
            return True
        elif isinstance(styles, dict):
            for n, v in styles.items():
                self.set(n, v)
            return True
        return False

    @property
    def is_empty(self) -> bool:
        return len(self) <= 0

    def __str__(self):
        return ''.join(f'{n}:{v};' for n, v in self.items())

    def output(self) -> str:
        return self.__str__()



from ...__templates__.styles import Styles


class Justify(Styles):
    def __init__(self, senior):
        super().__init__(senior, [
            'align_items_start',
            'align_items_end',
            'align_items_center',
            'align_items_baseline',
            'align_items_stretch',
            'align_items_sm_start',
            'align_items_sm_end',
            'align_items_sm_center',
            'align_items_sm_baseline',
            'align_items_sm_stretch',
            'align_items_md_start',
            'align_items_md_end',
            'align_items_md_center',
            'align_items_md_baseline',
            'align_items_md_stretch',
            'align_items_lg_start',
            'align_items_lg_end',
            'align_items_lg_center',
            'align_items_lg_baseline',
            'align_items_lg_stretch',
            'align_items_xl_start',
            'align_items_xl_end',
            'align_items_xl_center',
            'align_items_xl_baseline',
            'align_items_xl_stretch',
            'align_items_xxl_start',
            'align_items_xxl_end',
            'align_items_xxl_center',
            'align_items_xxl_baseline',
            'align_items_xxl_stretch',
        ])

    def set_align_items_start(self): self.set('align-items-start')

    def set_align_items_end(self): self.set('align-items-end')

    def set_align_items_center(self): self.set('align-items-center')

    def set_align_items_baseline(self): self.set('align-items-baseline')

    def set_align_items_stretch(self): self.set('align-items-stretch')

    def set_align_items_sm_start(self): self.set('align-items-sm-start')

    def set_align_items_sm_end(self): self.set('align-items-sm-end')

    def set_align_items_sm_center(self): self.set('align-items-sm-center')

    def set_align_items_sm_baseline(self): self.set('align-items-sm-baseline')

    def set_align_items_sm_stretch(self): self.set('align-items-sm-stretch')

    def set_align_items_md_start(self): self.set('align-items-md-start')

    def set_align_items_md_end(self): self.set('align-items-md-end')

    def set_align_items_md_center(self): self.set('align-items-md-center')

    def set_align_items_md_baseline(self): self.set('align-items-md-baseline')

    def set_align_items_md_stretch(self): self.set('align-items-md-stretch')

    def set_align_items_lg_start(self): self.set('align-items-lg-start')

    def set_align_items_lg_end(self): self.set('align-items-lg-end')

    def set_align_items_lg_center(self): self.set('align-items-lg-center')

    def set_align_items_lg_baseline(self): self.set('align-items-lg-baseline')

    def set_align_items_lg_stretch(self): self.set('align-items-lg-stretch')

    def set_align_items_xl_start(self): self.set('align-items-xl-start')

    def set_align_items_xl_end(self): self.set('align-items-xl-end')

    def set_align_items_xl_center(self): self.set('align-items-xl-center')

    def set_align_items_xl_baseline(self): self.set('align-items-xl-baseline')

    def set_align_items_xl_stretch(self): self.set('align-items-xl-stretch')

    def set_align_items_xxl_start(self): self.set('align-items-xxl-start')

    def set_align_items_xxl_end(self): self.set('align-items-xxl-end')

    def set_align_items_xxl_center(self): self.set('align-items-xxl-center')

    def set_align_items_xxl_baseline(self): self.set('align-items-xxl-baseline')

    def set_align_items_xxl_stretch(self): self.set('align-items-xxl-stretch')



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages
setup(
    name='hebill',
    version='3.1.1',
    description='Hebill Python Library',
    long_description=open(r'README.MD', encoding='utf-8').read(),
    long_description_content_type='text/plain',
    packages=find_packages(include=[
        'hebill',
        'hebill.*',
    ]),
    package_data={
        'hebill': ['*.md', '*.MD'],
    },
    install_requires=[
        'colorama==0.4.6',
        'numpy==1.26.4',
        'wxPython==4.2.1',
        'pillow==10.3.0',
    ],
    python_requires='>=3.12',
)



from ...__templates__.styles import Styles


class Justify(Styles):
    def __init__(self, senior):
        super().__init__(senior, [
            'flex_fill',
            'flex_sm_fill',
            'flex_md_fill',
            'flex_lg_fill',
            'flex_xl_fill',
            'flex_xxl_fill',

        ])

    def flex_fill(self): self.set('flex-fill')

    def flex_sm_fill(self): self.set('flex-sm-fill')

    def flex_md_fill(self): self.set('flex-md-fill')

    def flex_lg_fill(self): self.set('flex-lg-fill')

    def flex_xl_fill(self): self.set('flex-xl-fill')

    def flex_xxl_fill(self): self.set('flex-xxl-fill')



from .classes.core import Classes
from .styles.core import Styles


class Attributes(dict):
    def __init__(self):
        super().__init__()
        self._classes: Classes = Classes(self)
        self._styles: Styles = Styles(self)

    def __setitem__(self, key: str, value):
        key = key.lower()
        if key == 'class':
            self.classes.set(value)
            print('验证classes' + value)
            super().__setitem__(key, '')
        elif key == 'style':
            self.styles.set(value)
            print('验证styles' + value)
            super().__setitem__(key, '')
        else:
            super().__setitem__(key, value)

    def __getitem__(self, key):
        key = key.lower()
        if key == 'class':
            return self.classes.output()
        elif key == 'style':
            return self.styles.output()
        else:
            return super().__getitem__(key)

    def is_empty(self) -> bool:
        return self.classes.is_empty and self.styles.is_empty and len(self) <= 0

    def set_id(self, uid: str):
        self['id'] = uid

    def get_id(self, auto=False):
        if not auto:
            return self.get('id')
        if not self.get('id'):
            from .....string.core import String
            self.set('id', String.random())
        return self.get('id')

    @property
    def classes(self):
        return self._classes

    @property
    def styles(self):
        return self._styles

    def __str__(self):
        if len(self.classes) > 0:
            super().__setitem__('class', self.classes.output())
        if len(self.styles) > 0:
            super().__setitem__('style', self.styles.output())
        return ''.join(f" {n}=\"{v}\"" for n, v in self.items())

    def set(self, name: str, value: str):
        if name:
            self[name] = value

    def output(self) -> str:
        return self.__str__()



class Document:
    def __init__(self):
        self._elements = {}
        from .components import Html
        self._html = Html(self)
        from .nodes.group.create.core import Create
        self._create = Create(self)

        self.titles: list = []
        self.title_delimiter: str = " > "
        self.output_break: bool = True
        self.output_retraction: str = "	"
        self.output_next_breakable: bool = True

    @property
    def elements(self): return self._elements

    def local_add_element(self, ele): self._elements[ele.id] = ele

    @property
    def html(self): return self._html

    def create(self): return self._create

    def output(self) -> str:
        if len(self.titles) > 0:
            self.html.head.title.content.text = self.title_delimiter.join(self.titles)
        s = "<!DOCTYPE html>"
        if self.output_break:
            s += "\n"
        s += self.html.output()
        return s



from ...__templates__.styles import Styles


class FlexRow(Styles):
    def __init__(self, senior):
        super().__init__(senior, [
            'flex-column',
            'flex-column-reverse',
            'flex-sm-column',
            'flex-sm-column-reverse',
            'flex-md-column',
            'flex-md-column-reverse',
            'flex-lg-column',
            'flex-lg-column-reverse',
            'flex-xl-column',
            'flex-xl-column-reverse',
            'flex-xxl-column',
            'flex-xxl-column-reverse',
            'flex-row',
            'flex-row-reverse',
            'flex-sm-row',
            'flex-sm-row-reverse',
            'flex-md-row',
            'flex-md-row-reverse',
            'flex-lg-row',
            'flex-lg-row-reverse',
            'flex-xl-row',
            'flex-xl-row-reverse',
            'flex-xxl-row',
            'flex-xxl-row-reverse',
        ])

    def set_flex_column(self): self.set('flex-column')
    def set_flex_column_reverse(self): self.set('d-flex-column-reverse')
    def set_flex_sm_column(self): self.set('flex-sm-column')
    def set_flex_sm_column_reverse(self): self.set('d-flex-sm-column-reverse')
    def set_flex_md_column(self): self.set('flex-md-column')
    def set_flex_md_column_reverse(self): self.set('d-flex-md-column-reverse')
    def set_flex_lg_column(self): self.set('flex-lg-column')
    def set_flex_lg_column_reverse(self): self.set('d-flex-lg-column-reverse')
    def set_flex_xl_column(self): self.set('flex-xl-column')
    def set_flex_xl_column_reverse(self): self.set('d-flex-xl-column-reverse')
    def set_flex_xxl_column(self): self.set('flex-xxl-column')
    def set_flex_xxl_column_reverse(self): self.set('d-flex-xxl-column-reverse')
    def set_flex_row(self): self.set('flex-row')
    def set_flex_row_reverse(self): self.set('d-flex-row-reverse')
    def set_flex_sm_row(self): self.set('flex-sm-row')
    def set_flex_sm_row_reverse(self): self.set('d-flex-sm-row-reverse')
    def set_flex_md_row(self): self.set('flex-md-row')
    def set_flex_md_row_reverse(self): self.set('d-flex-md-row-reverse')
    def set_flex_lg_row(self): self.set('flex-lg-row')
    def set_flex_lg_row_reverse(self): self.set('d-flex-lg-row-reverse')
    def set_flex_xl_row(self): self.set('flex-xl-row')
    def set_flex_xl_row_reverse(self): self.set('d-flex-xl-row-reverse')
    def set_flex_xxl_row(self): self.set('flex-xxl-row')
    def set_flex_xxl_row_reverse(self): self.set('d-flex-xxl-row-reverse')



from ...nodes import Tag
from ...__extensions__.toggle.core import Toggle


class Button(Tag, Toggle):
    def __init__(self, senior, text=None):
        super().__init__(senior, 'button')
        self.add_junior(text)



from ...nodes import Tag


class Script(Tag):
    def __init__(self, senior, url: str = None):
        super().__init__(senior, 'script')
        if url is not None:
            self.attributes["src"] = url



import base64
import io
from .constants import *


class PNG:
    def __init__(self, source):
        if source.endswith(".png"):
            with open(source, 'rb') as file:
                self._source = base64.b64encode(file.read()).decode('utf-8')
        else:
            self._source = source

    @property
    def source(self): return self._source

    @property
    def file(self): return io.BytesIO(self.bites)

    @property
    def bites(self): return base64.b64decode(self._source.encode('utf-8'))

    def _resize(self, width: int = None, height: int = None):
        pass

    # 使用 PIL.Image
    def pil_image(self, width: int = None, height: int = None):
        from PIL import Image
        image = Image.open(self.file)
        if width:
            image = image.resize((width, height if height else width))
        return image

    def pil_image_save(self, file: str = None, width: int = None, height: int = None):
        self.pil_image(width, height).save(file)

    def pil_icons(self, file: str = None, sizes: list | tuple = None):
        """
        :param file:
        :param sizes: 默认尺寸：(16, 24, 32, 48, 64, 128, 256)
        :return: 保存多尺寸图标文件
        """
        sizes = sizes if sizes else (16, 24, 32, 48, 64, 128, 256)
        file = file if file else f'icon-{'-'.join(map(str, sizes))}.ico'
        image = self.pil_image()
        image.save(file, format='ICO', sizes=[(size, size) for size in sizes])

    def pil_icons_save(self, file: str = None, sizes: list | tuple = None):
        """
        :param file:
        :param sizes: 默认尺寸：(16, 24, 32, 48, 64, 128, 256)
        :return: 保存多尺寸图标文件, 无需
        """
        sizes = sizes if sizes else (16, 24, 32, 48, 64, 128, 256)
        file = file if file else f'icon-{'-'.join(map(str, sizes))}.ico'
        image = self.pil_image()
        image.save(file, format='ICO', sizes=[(size, size) for size in sizes])

    # 使用 wx.Image
    def wx_image(self, width: int = None, height: int = None):
        import wx
        image = wx.Image(self.file)
        if width:
            image = image.Scale(width, height if height else width, wx.IMAGE_QUALITY_HIGH)
        return image

    def wx_image_save(self, file: str = None, width: int = None, height: int = None):
        self.wx_image(width, height).SaveFile(file)

    # 使用 wx.Bitmap
    def wx_bitmap(self, width: int = None, height: int = None):
        import wx
        image = wx.Image(self.file, wx.BITMAP_TYPE_ANY)
        bitmap = wx.Bitmap(image)
        if width:
            image = bitmap.ConvertToImage().Scale(width, width if height is None else height, wx.IMAGE_QUALITY_HIGH)
        return wx.Bitmap(image)

    def wx_bitmap_save(self, file: str = None, width: int = None, height: int = None):
        import wx
        self.wx_bitmap(width, height).SaveFile(file, wx.BITMAP_TYPE_PNG)

    def wx_bitmap_io(self, width: int = None, height: int = None):
        import wx
        buffer = io.BytesIO()
        self.wx_bitmap(width, height).SaveFile(buffer, wx.BITMAP_TYPE_PNG)
        return buffer.getvalue()

    # 使用 wx.Icon
    def wx_icon(self, size: int = None):
        import wx
        icon = wx.Icon()
        icon.CopyFromBitmap(self.wx_image(size if size else None).ConvertToBitmap())
        return icon

    def wx_icon_save(self, file: str = None, size: int = None):
        import wx
        image = self.wx_image(size)
        image.SaveFile(file if file else f'wx-icon-{image.GetWidth()}x{image.GetHeight()}', wx.BITMAP_TYPE_PNG)

    def wx_icons(self, sizes: list = None):
        sizes = sizes if sizes else (16, 24, 32, 48, 64, 128, 256)
        image = self.pil_image()
        ico_data = io.BytesIO()
        image.save(ico_data, format='ICO', sizes=[(size, size) for size in sizes])
        # 使用 wx.Image 创建位图对象
        import wx
        wx_image = wx.Image(ico_data, type=wx.BITMAP_TYPE_ICO)
        # 创建 wx.Icon 对象
        icon = wx.Icon()
        icon.CopyFromBitmap(wx_image.ConvertToBitmap())
        return icon


class PNGIconClose(PNG):
    def __init__(self):
        super().__init__(ICO_CLOSE)


class PNGIconDemo(PNG):
    def __init__(self):
        super().__init__(ICO_DEMO)


class PNGIconDocument(PNG):
    def __init__(self):
        super().__init__(ICO_DOCUMENT)


class PNGIconHelp(PNG):
    def __init__(self):
        super().__init__(ICO_HELP)


class PNGIconHome(PNG):
    def __init__(self):
        super().__init__(ICO_HOME)


class PNGIconIcon(PNG):
    def __init__(self):
        super().__init__(ICO_ICON)


class PNGIconInfo(PNG):
    def __init__(self):
        super().__init__(ICO_INFO)


class PNGIconLangauge(PNG):
    def __init__(self):
        super().__init__(ICO_LANGUAGE)


class PNGIconMaximize(PNG):
    def __init__(self):
        super().__init__(ICO_MAXIMIZE)


class PNGIconMinimize(PNG):
    def __init__(self):
        super().__init__(ICO_MINIMIZE)


class PNGIconMore(PNG):
    def __init__(self):
        super().__init__(ICO_MORE)


class PNGIconNormalize(PNG):
    def __init__(self):
        super().__init__(ICO_NORMALIZE)


class PNGIconQuite(PNG):
    def __init__(self):
        super().__init__(ICO_QUITE)


class PNGIconSetting(PNG):
    def __init__(self):
        super().__init__(ICO_SETTING)


class PNGIconUser(PNG):
    def __init__(self):
        super().__init__(ICO_USER)


class PNGIconHebill(PNG):
    def __init__(self):
        super().__init__(ICO_HEBILL)


class PNGIconYangs(PNG):
    def __init__(self):
        super().__init__(ICO_YANGS)



from ..node.core import Node


class Content(Node):
    def __init__(self, sir, text: str = None):
        super().__init__(sir)
        self._text = text if text is not None else ''

    @property
    def text(self):
        return self._text

    @text.setter
    def text(self, text):
        self._text = text

    def output(self):
        self.document.output_next_breakable = False
        return self.text.replace("&", "&amp;").replace("<", "&lt;").replace(">", "&gt;")



from ....tags.head.core import Head as HeadParentClass
from .title.core import Title


class Head(HeadParentClass):
    def __init__(self, senior):
        super().__init__(senior)
        self._metas = self.create.node.group()
        self._title = self.create.tag.title()
        self._libraries = self.create.node.group()

    @property
    def metas(self):
        return self._metas.create.tag.title()

    @property
    def libraries(self):
        return self._libraries

    @property
    def title(self):
        return self._title.content

    @title.setter
    def title(self, title):
        self._title.content = title




from ...__templates__.styles import Styles


class Size(Styles):
    def __init__(self, senior):
        super().__init__(senior, [
            'container',
            'container-sm',
            'container-md',
            'container-lg',
            'container-xl',
            'container-xxl',
            'container-fluid',
        ])

    def set_container(self): return self.set('container')

    def set_container_small(self): return self.set('container-sm')

    def set_container_middle(self): return self.set('container-md')

    def set_container_large(self): return self.set('container-lg')

    def set_container_x_large(self): return self.set('container-xl')

    def set_container_xx_large(self): return self.set('container-xxl')

    def set_container_fluid(self): return self.set('container-fluid')



from .png.core import PNG
from .png.core import PNGIconClose
from .png.core import PNGIconDemo
from .png.core import PNGIconDocument
from .png.core import PNGIconHelp
from .png.core import PNGIconHome
from .png.core import PNGIconIcon
from .png.core import PNGIconInfo
from .png.core import PNGIconLangauge
from .png.core import PNGIconMaximize
from .png.core import PNGIconMinimize
from .png.core import PNGIconMore
from .png.core import PNGIconNormalize
from .png.core import PNGIconQuite
from .png.core import PNGIconSetting
from .png.core import PNGIconUser

from .png.core import PNGIconHebill
from .png.core import PNGIconYangs



from ..group.core import Group


class Tag(Group):
    def __init__(self, sir, tag):
        super().__init__(sir)
        self._tag = tag
        from .attributes.core import Attributes
        self._attributes = Attributes()
        self.output_breakable = True
        self.output_paired = True

    @property
    def tag(self): return self._tag

    @property
    def attributes(self): return self._attributes

    def output(self):
        s = ""
        if self.document.output_break:
            if self.output_breakable and self.document.output_next_breakable:
                if self.level > 0:
                    s += "\n"
            s += self.document.output_retraction * self.level
        s += "<" + self.tag
        s += self.attributes.output()
        s += ">"
        if not self.output_paired:
            return s
        self.document.output_next_breakable = True
        si = super().output()
        s += si
        if self.document.output_break:
            if si != "" and self.document.output_next_breakable:
                s += "\n" + "	" * self.level
        s += "</" + self.tag + ">"
        self.document.output_next_breakable = True
        return s



class Toggle:
    def set_toggle_target_by_id(self, tag, show=False):
        self.attributes.set('data-bs-toggle', 'collapse')
        self.attributes.set('aria-expanded', 'true' if show else 'false')
        self.attributes.set('data-bs-target', f'#{tag.attributes.get_id(True)}')
        self.attributes.set('aria-controls', tag.attributes.get_id())
        tag.attributes.classes.set('collapse')

    def set_toggle_target_by_class(self, tag, show=False):
        self.attributes.set('data-bs-toggle', 'collapse')
        self.attributes.set('aria-expanded', 'true' if show else 'false')
        self.attributes.set('data-bs-target', f'#{tag.attributes.get_id(True)}')
        self.attributes.set('aria-controls', tag.attributes.get_id())
        tag.attributes.classes.set('collapse')



from ...tags.table.core import Table as TableParentClass
from .thead.core import Thead
from .tbody.core import Tbody


class Table(TableParentClass):
    def __init__(self, senior):
        super().__init__(senior)
        self._head_wrap = self.create.node.group()
        self._head = None
        self._body = None
        self.attributes.classes.set('table')

        self._style = None

    @property
    def head(self) -> Thead:
        if self._head is None:
            self._head = Thead(self._head_wrap)
        return self._head

    @property
    def head_rows(self): return self.head.rows

    @property
    def head_row(self): return self.head.row

    def add_head_row(self, data: list = None): return self.head.add_row(data)

    @property
    def head_cells(self): return self.head.cells

    @property
    def head_cell(self): return self.head.cell

    def add_head_cell(self, text=None): return self.head.add_cell(text)

    @property
    def body(self) -> Tbody:
        if self._body is None:
            self._body = Tbody(self)
        return self._body

    @property
    def body_rows(self): return self.body.rows

    @property
    def body_row(self): return self.body.row

    def add_body_row(self, data: list = None): return self.body.add_row(data)

    @property
    def body_cells(self): return self.body.cells

    @property
    def body_cell(self): return self.body.cell

    def add_body_cell(self, text=None): return self.body.add_cell(text)

    @property
    def color(self):
        if self._style is None:
            from .__templates__.color import Color
            self._style = Color(self)
        return self._style

    def set_striped(self):
        return self.attributes.classes.set('table-striped')

    def set_striped_columns(self):
        return self.attributes.classes.set('table-striped-columns')

    def set_hover(self):
        return self.attributes.classes.set('table-hover')

    def set_bordered(self):
        self.attributes.classes.unset('table-borderless')
        return self.attributes.classes.set('table-bordered')

    def set_borderless(self):
        self.attributes.classes.unset('table-bordered')
        return self.attributes.classes.set('table-borderless')

    def set_small(self):
        return self.attributes.classes.set('table-sm')



from ...nodes.node.core import Node


class Code(Node):
    def __init__(self, sir, text: str = ''):
        super().__init__(sir)
        self.text = text

    def output(self):
        self.document.output_end_breakable = False
        return f"{self.text}"



from ...nodes import Tag


class Meta(Tag):
    def __init__(self, senior):
        super().__init__(senior, 'meta')
        self.output_paired = False

    def set_attribute_charset(self, charset):
        self.attributes['charset'] = charset

    def set_attribute_name(self, name):
        self.attributes['name'] = name

    def set_attribute_content(self, content):
        self.attributes['content'] = content



from ...nodes import Tag


class Title(Tag):
    def __init__(self, senior, text: str = None):
        super().__init__(senior, 'title')
        self.output_break_inner = False
        self._content = self.create.node.content()
        self.add_junior(text)

    @property
    def content(self):
        return self._content.text

    @content.setter
    def content(self, content):
        self._content.text = content



from ...__templates__.styles import Styles


class Size(Styles):
    def __init__(self, senior):
        super().__init__(senior, [
            'btn-sm',
            'btn-lg',
        ])

    def set_small(self): return self.set('btn-sm')

    def set_large(self): return self.set('btn-lg')



# -*- coding: utf-8 -*-
NAME = 'hebill'
VERSION = '3.1.1'
DESCRIPTION = 'Hebill Python Library'
LONG_DESCRIPTION = 'README.MD'
LONG_DESCRIPTION_TYPE = 'text/markdown'
URL = 'http://www.hebill.net/'
AUTHOR = 'He Bill'
AUTHOR_EMAIL = 'hebill@hebill.net'
AUTHOR_URL = 'http://www.hebill.net/'



from .html.core import Html
from .alert.core import Alert
from .breadcrumb.core import Breadcrumb
from .button.core import Button
from .container.core import Container
from .pagination.core import Pagination
from .table.core import Table





class Libraries:
    def __init__(self, htm):
        from ....components.html.core import Html
        self._htm: Html = htm
        self._files = []

    def add_link_js(self, url):
        if url not in self._files:
            self._files.append(url)
        pass

    def add_link_css(self, url):
        if url not in self._files:
            self._files.append(url)
        pass

    def add_meta(self):
        pass




from ...__templates__.styles import Styles


class Justify(Styles):
    def __init__(self, senior):
        super().__init__(senior, [
            'align_self_start',
            'align_self_end',
            'align_self_center',
            'align_self_baseline',
            'align_self_stretch',
            'align_self_sm_start',
            'align_self_sm_end',
            'align_self_sm_center',
            'align_self_sm_baseline',
            'align_self_sm_stretch',
            'align_self_md_start',
            'align_self_md_end',
            'align_self_md_center',
            'align_self_md_baseline',
            'align_self_md_stretch',
            'align_self_lg_start',
            'align_self_lg_end',
            'align_self_lg_center',
            'align_self_lg_baseline',
            'align_self_lg_stretch',
            'align_self_xl_start',
            'align_self_xl_end',
            'align_self_xl_center',
            'align_self_xl_baseline',
            'align_self_xl_stretch',
            'align_self_xxl_start',
            'align_self_xxl_end',
            'align_self_xxl_center',
            'align_self_xxl_baseline',
            'align_self_xxl_stretch',
        ])

    def align_self_start(self): self.set('align-self-start')

    def align_self_end(self): self.set('align-self-end')

    def align_self_center(self): self.set('align-self-center')

    def align_self_baseline(self): self.set('align-self-baseline')

    def align_self_stretch(self): self.set('align-self-stretch')

    def align_self_sm_start(self): self.set('align-self-sm-start')

    def align_self_sm_end(self): self.set('align-self-sm-end')

    def align_self_sm_center(self): self.set('align-self-sm-center')

    def align_self_sm_baseline(self): self.set('align-self-sm-baseline')

    def align_self_sm_stretch(self): self.set('align-self-sm-stretch')

    def align_self_md_start(self): self.set('align-self-md-start')

    def align_self_md_end(self): self.set('align-self-md-end')

    def align_self_md_center(self): self.set('align-self-md-center')

    def align_self_md_baseline(self): self.set('align-self-md-baseline')

    def align_self_md_stretch(self): self.set('align-self-md-stretch')

    def align_self_lg_start(self): self.set('align-self-lg-start')

    def align_self_lg_end(self): self.set('align-self-lg-end')

    def align_self_lg_center(self): self.set('align-self-lg-center')

    def align_self_lg_baseline(self): self.set('align-self-lg-baseline')

    def align_self_lg_stretch(self): self.set('align-self-lg-stretch')

    def align_self_xl_start(self): self.set('align-self-xl-start')

    def align_self_xl_end(self): self.set('align-self-xl-end')

    def align_self_xl_center(self): self.set('align-self-xl-center')

    def align_self_xl_baseline(self): self.set('align-self-xl-baseline')

    def align_self_xl_stretch(self): self.set('align-self-xl-stretch')

    def align_self_xxl_start(self): self.set('align-self-xxl-start')

    def align_self_xxl_end(self): self.set('align-self-xxl-end')

    def align_self_xxl_center(self): self.set('align-self-xxl-center')

    def align_self_xxl_baseline(self): self.set('align-self-xxl-baseline')

    def align_self_xxl_stretch(self): self.set('align-self-xxl-stretch')



from ...nodes import Tag


class Link(Tag):
    def __init__(self, senior, url: str = None):
        super().__init__(senior, 'link')
        if url is not None:
            self.attributes["href"] = url



CN_INTEGER_0_TO_9 = '零壹贰叁肆伍陆柒捌玖'
CN_POINTS_UNITS = '角分厘'
CN_DECIMAL_4S_UNITS = '万亿兆京垓秭穰沟涧正载极'
CN_DECIMAL_UNITS = '拾佰仟'

CN_CURRENCY_UNIT = "圆"
CN_POINT_NAME = '点'
CN_ONLY_NAME = '整'
CN_CNY_NAME = '人民币'
CN_USD_NAME = '美元'
CN_EUR_NAME = '欧元'
CN_MINUS = '负'

EN_CNY_NAME = 'Say Chinese Yuan'
EN_USD_NAME = 'Say U.S.Dollar'
EN_EUR_NAME = 'Say Euro'
EN_ONLY_NAME = 'Only'
EN_POINT_NAME = 'Point'
EN_AND_NAME = 'and'
EN_MINUS = 'Minus'


EN_INTEGER_0_TO_9 = ['Zero', 'One', 'Two', 'Three', 'Four', 'Five', 'Six', 'Seven', 'Eight', 'Nine']
EN_INTEGER_1NS = ['Ten', 'Eleven', 'Twelve', 'Thirteen', 'Fourteen', 'Fifteen', 'Sixteen', 'Seventeen',
                  'Eighteen', 'Nineteen']
EN_INTEGER_2NS = ['Twenty', 'Thirty', 'Forty', 'Fifty', 'Sixty', 'Seventy', 'Eighty', 'Ninety']
EN_HUNDRED_NAME = 'Hundred'
EN_DECIMAL_3S_UNITS = ['Thousand', 'Million', 'Billion', 'Trillion', 'Quadrillion', 'Quintillion', 'Sextillion',
                       'Septillion', 'Octillion', 'Nonillion', 'Decillion']
EN_POINTS_UNITS = ['Dime', 'Cent', 'Mill']



class Tags:
    def __init__(self, sir):
        self.senior = sir

    def a(self, title: str = None, url: str = None):
        from .....tags import A
        return A(self.senior, title, url)

    def body(self):
        from .....tags import Body
        return Body(self.senior)

    def button(self, text):
        from .....tags import Button
        return Button(self.senior, text)

    def div(self, text: str = None):
        from .....tags import Div
        return Div(self.senior, text)

    def h1(self, text: str = None):
        from .....tags import H1
        return H1(self.senior, text)

    def h2(self, text: str = None):
        from .....tags import H2
        return H2(self.senior, text)

    def h3(self, text: str = None):
        from .....tags import H3
        return H3(self.senior, text)

    def h4(self, text: str = None):
        from .....tags import H4
        return H4(self.senior, text)

    def h5(self, text: str = None):
        from .....tags import H5
        return H5(self.senior, text)

    def h6(self, text: str = None):
        from .....tags import H6
        return H6(self.senior, text)

    def head(self):
        from .....tags import Head
        return Head(self.senior)

    def html(self, lang: str = None):
        from .....tags import Html
        return Html(self.senior, lang)

    def input_text(self, name: str = None, value: str | int | float = None, placeholder: str = None):
        from .....tags import InputText
        return InputText(self.senior, name, value, placeholder)

    def li(self, text: str = None):
        from .....tags import Li
        return Li(self.senior, text)

    def link(self, url: str = None):
        from .....tags import Link
        return Link(self.senior, url)

    def nav(self):
        from .....tags import Nav
        return Nav(self.senior)

    def meta(self):
        from .....tags import Meta
        return Meta(self.senior)

    def ol(self):
        from .....tags import Ol
        return Ol(self.senior)

    def script(self, url: str = None):
        from .....tags import Script
        return Script(self.senior, url)

    def span(self, text: str = None):
        from .....tags import Span
        return Span(self.senior, text)

    def table(self):
        from .....tags import Table
        return Table(self.senior)

    def tbody(self):
        from .....tags import Tbody
        return Tbody(self.senior)

    def th(self):
        from .....tags import Th
        return Th(self.senior)

    def td(self):
        from .....tags import Td
        return Td(self.senior)

    def thead(self):
        from .....tags import Thead
        return Thead(self.senior)

    def title(self, text: str = None):
        from .....tags import Title
        return Title(self.senior, text)

    def tr(self):
        from .....tags import Tr
        return Tr(self.senior)

    def ul(self):
        from .....tags import Ul
        return Ul(self.senior)



from ...tags import Nav


class Pagination(Nav):
    def __init__(self, senior):
        super().__init__(senior)
        self.attributes.set('aria-label', 'pagination')
        self._ul = self.create.tag.ul()
        self._ul.attributes.classes.set('pagination')
        self._size = None

    def add_item(self, title=None, url=None, active=False):
        li = self._ul.create.tag.li()
        li.attributes.classes.set('page-item')
        a = li.create.tag.a(title, url)
        a.attributes.classes.set('page-link')
        if active:
            li.attributes.classes.set('active')
            li.attributes.set('aria-current', 'page')
        return self

    @property
    def size(self):
        if self._size is None:
            from .size import Size
            self._size = Size(self._ul)
        return self._size



from ....tags import Ul


class Navbar(Ul):
    def __init__(self, senior):
        super().__init__(senior)
        self.attributes.classes.set('navbar')
        self._div = self.create.tag.div()
        self._div.attributes.classes.set('container-fluid')
        self._brand_wrap = self._div.create.node.group()
        self._brand = None


from ...tags.div.core import Div


class Alert(Div):
    def __init__(self, senior, text=None):
        super().__init__(senior, text)
        self.attributes.classes.set('alert')

        self._color = None

    @property
    def color(self):
        if self._color is None:
            from .color import Color
            self._color = Color(self)
        return self._color



class Styles:
    def __init__(self, senior, styles: list):
        self._senior = senior
        self._styles = styles

    def set(self, style: str | int) -> bool:
        if isinstance(style, int):
            if not 0 <= style < len(self._styles):
                return False
            style = self._styles[style]
        if style not in self._styles:
            return False
        self._senior.attributes.classes.unset(self._styles)
        return self._senior.attributes.classes.set(style)



from ...tags import Nav


class Navbar(Nav):
    def __init__(self, senior):
        super().__init__(senior)
        self.attributes.classes.set('navbar')
        self._div = self.create.tag.div()
        self._div.attributes.classes.set('container-fluid')
        self._brand_wrap = self._div.create.node.group()
        self._brand = None
        btn = self._div.create.tag.div()
        btn.attributes.classes.set('navbar-toggler')
        btn.attributes.set('tyre', 'button')
        btn.attributes.set('data-bs-toggle', 'collapse')

    @property
    def brand(self): return self._brand

    @brand.setter
    def brand(self, text: str): self._brand = text

    def output(self):
        if self.brand:
            pass
        super().output()



from ...__templates__.styles import Styles


class Justify(Styles):
    def __init__(self, senior):
        super().__init__(senior, [
            'justify-content-start',
            'justify-content-end',
            'justify-content-center',
            'justify-content-between',
            'justify-content-around',
            'justify-content-evenly',
            'justify-content-sm-start',
            'justify-content-sm-end',
            'justify-content-sm-center',
            'justify-content-sm-between',
            'justify-content-sm-around',
            'justify-content-sm-evenly',
            'justify-content-md-start',
            'justify-content-md-end',
            'justify-content-md-center',
            'justify-content-md-between',
            'justify-content-md-around',
            'justify-content-md-evenly',
            'justify-content-lg-start',
            'justify-content-lg-end',
            'justify-content-lg-center',
            'justify-content-lg-between',
            'justify-content-lg-around',
            'justify-content-lg-evenly',
            'justify-content-xl-start',
            'justify-content-xl-end',
            'justify-content-xl-center',
            'justify-content-xl-between',
            'justify-content-xl-around',
            'justify-content-xl-evenly',
            'justify-content-xxl-start',
            'justify-content-xxl-end',
            'justify-content-xxl-center',
            'justify-content-xxl-between',
            'justify-content-xxl-around',
            'justify-content-xxl-evenly',
        ])

    def set_justify_content_start(self): self.set('justify_content_start')

    def set_justify_content_end(self): self.set('justify_content_end')

    def set_justify_content_center(self): self.set('justify_content_center')

    def set_justify_content_between(self): self.set('justify_content_between')

    def set_justify_content_around(self): self.set('justify_content_around')

    def set_justify_content_evenly(self): self.set('justify_content_evenly')

    def set_justify_content_sm_start(self): self.set('justify_content_sm_start')

    def set_justify_content_sm_end(self): self.set('justify_content_sm_end')

    def set_justify_content_sm_center(self): self.set('justify_content_sm_center')

    def set_justify_content_sm_between(self): self.set('justify_content_sm_between')

    def set_justify_content_sm_around(self): self.set('justify_content_sm_around')

    def set_justify_content_sm_evenly(self): self.set('justify_content_sm_evenly')

    def set_justify_content_md_start(self): self.set('justify_content_md_start')

    def set_justify_content_md_end(self): self.set('justify_content_md_end')

    def set_justify_content_md_center(self): self.set('justify_content_md_center')

    def set_justify_content_md_between(self): self.set('justify_content_md_between')

    def set_justify_content_md_around(self): self.set('justify_content_md_around')

    def set_justify_content_md_evenly(self): self.set('justify_content_md_evenly')

    def set_justify_content_lg_start(self): self.set('justify_content_lg_start')

    def set_justify_content_lg_end(self): self.set('justify_content_lg_end')

    def set_justify_content_lg_center(self): self.set('justify_content_lg_center')

    def set_justify_content_lg_between(self): self.set('justify_content_lg_between')

    def set_justify_content_lg_around(self): self.set('justify_content_lg_around')

    def set_justify_content_lg_evenly(self): self.set('justify_content_lg_evenly')

    def set_justify_content_xl_start(self): self.set('justify_content_xl_start')

    def set_justify_content_xl_end(self): self.set('justify_content_xl_end')

    def set_justify_content_xl_center(self): self.set('justify_content_xl_center')

    def set_justify_content_xl_between(self): self.set('justify_content_xl_between')

    def set_justify_content_xl_around(self): self.set('justify_content_xl_around')

    def set_justify_content_xl_evenly(self): self.set('justify_content_xl_evenly')

    def set_justify_content_xxl_start(self): self.set('justify_content_xxl_start')

    def set_justify_content_xxl_end(self): self.set('justify_content_xxl_end')

    def set_justify_content_xxl_center(self): self.set('justify_content_xxl_center')

    def set_justify_content_xxl_between(self): self.set('justify_content_xxl_between')

    def set_justify_content_xxl_around(self): self.set('justify_content_xxl_around')

    def set_justify_content_xxl_evenly(self): self.set('justify_content_xxl_evenly')




from ...__templates__.styles import Styles


class Flex(Styles):
    def __init__(self, senior):
        super().__init__(senior, [
            'd-flex',
            'd-inline-flex',
            'd-sm-flex',
            'd-sm-inline-flex',
            'd-md-flex',
            'd-md-inline-flex',
            'd-lg-flex',
            'd-lg-inline-flex',
            'd-xl-flex',
            'd-xl-inline-flex',
            'd-xxl-flex',
            'd-xxl-inline-flex',
        ])

    def set_d_flex(self): self.set('d-flex')
    def set_d_inline_flex(self): self.set('d-inline-flex')
    def set_d_sm_flex(self): self.set('d-sm-flex')
    def set_d_sm_inline_flex(self): self.set('d-sm-inline-flex')
    def set_d_md_flex(self): self.set('d-md-flex')
    def set_d_md_inline_flex(self): self.set('d-md-inline-flex')
    def set_d_lg_flex(self): self.set('d-md-lg-flex')
    def set_d_lg_inline_flex(self): self.set('d-lg-inline-flex')
    def set_d_xl_flex(self): self.set('d-md-xl-flex')
    def set_d_xl_inline_flex(self): self.set('d-xl-inline-flex')
    def set_d_xxl_flex(self): self.set('d-md-xxl-flex')
    def set_d_xxl_inline_flex(self): self.set('d-xxl-inline-flex')



from .a.core import A
from .body.core import Body
from .button.core import Button
from .div.core import Div
from .h1.core import H1
from .h2.core import H2
from .h3.core import H3
from .h4.core import H4
from .h5.core import H5
from .h6.core import H6
from .head.core import Head
from .html.core import Html
from .input_text.core import InputText
from .li.core import Li
from .link.core import Link
from .meta.core import Meta
from .nav.core import Nav
from .ol.core import Ol
from .script.core import Script
from .span.core import Span
from .table.core import Table
from .tbody.core import Tbody
from .td.core import Td
from .th.core import Th
from .thead.core import Thead
from .title.core import Title
from .tr.core import Tr
from .ul.core import Ul



class Node:
    def __init__(self, sir):
        self._id = id(self)
        from ...core import Document
        self._document: Document
        from ..group.core import Group
        self._senior: Group | None = None
        if isinstance(sir, Document):
            self._document = sir
        elif isinstance(sir, Group):
            self._senior = sir
            self._document = sir.document
            self.senior.local_add_junior(self)
        self.document.local_add_element(self)
        self.output_breakable = False

    @property
    def id(self): return self._id

    @property
    def document(self): return self._document

    @property
    def senior(self): return self._senior

    def local_change_senior(self, sir):
        self.senior.local_remove_junior(self)
        self._senior = sir

    @property
    def level(self) -> int:
        if self._senior is None:
            return 0
        from ...nodes.tag.core import Tag
        from ...nodes.group.core import Group
        if isinstance(self, Group) and not isinstance(self, Tag):
            return self.senior.level
        return self.senior.level + 1

    def output(self) -> str:
        pass



from ....html.tags import Nav, A


class Breadcrumb(Nav):
    def __init__(self, senior):
        super().__init__(senior)
        self.attributes.set('aria-label', 'breadcrumb')
        self._ol = self.create.tag.ol()
        self._ol.attributes.classes.set('breadcrumb')

    def add_item(self, title=None, url=None, active=False):
        li = self._ol.create.tag.li()
        li.attributes.classes.set('breadcrumb-item')
        if not active:
            li.create.tag.a(title, url)
        else:
            li.create.node.content(title)
            li.attributes.classes.set('active')
            li.attributes.set('aria-current', 'page')

    def set_divider(self, divider: str):
        self.attributes.styles.set('--bs-breadcrumb-divider', f'\'{divider}\'')



from ..node.core import Node


class Group(Node):
    def __init__(self, sir):
        super().__init__(sir)
        self._juniors = {}
        self._create = None

    @property
    def juniors(self): return self._juniors

    def add_junior(self, jun=None):
        if jun is None:
            return False
        if isinstance(jun, str):
            return self.create.node.content(jun)
        if isinstance(jun, Node):
            jun.local_change_senior(self)
            self._juniors[jun.id] = jun
            return True
        return self.create.node.content(str(jun))

    def local_add_junior(self, jun: Node):
        self._juniors[jun.id] = jun

    def local_remove_junior(self, jun):
        if jun.id in self._juniors.keys():
            del self._juniors[jun.id]

    def has_juniors(self): return len(self._juniors) > 0

    @property
    def create(self):
        if self._create is None:
            from .create.core import Create
            self._create = Create(self)
        return self._create

    def output(self):
        s = ""
        if self.has_juniors():
            for key, value in self.juniors.items():
                s += value.output()
        return s



class Components:
    def __init__(self, sir):
        self.senior = sir

    def alert(self, text):
        from .....components import Alert
        return Alert(self.senior, text)

    def breadcrumb(self):
        from .....components import Breadcrumb
        return Breadcrumb(self.senior)

    def button(self, text):
        from .....components import Button
        return Button(self.senior, text)

    def container(self):
        from .....components import Container
        return Container(self.senior)

    def pagination(self):
        from .....components import Pagination
        return Pagination(self.senior)

    def table(self):
        from .....components import Table
        return Table(self.senior)



import re
from .constants import CN_DECIMAL_4S_UNITS, EN_DECIMAL_3S_UNITS, CN_INTEGER_0_TO_9, CN_DECIMAL_UNITS, CN_POINT_NAME, \
    CN_MINUS, CN_CURRENCY_UNIT, CN_POINTS_UNITS, CN_ONLY_NAME, CN_CNY_NAME, CN_USD_NAME, CN_EUR_NAME, EN_INTEGER_0_TO_9, \
    EN_INTEGER_1NS, EN_INTEGER_2NS, EN_HUNDRED_NAME, EN_POINT_NAME, EN_MINUS, EN_POINTS_UNITS, EN_CNY_NAME, EN_USD_NAME, \
    EN_EUR_NAME, EN_ONLY_NAME, EN_AND_NAME


class Digital(float):
    def __new__(cls, value):
        if not isinstance(value, (int, float)):
            raise TypeError(f"输入变量必须为整形或浮点型: {value}")
        return super().__new__(cls, value)

    def __init__(self, value):
        self._decimal_string = None
        self._integer_string = None
        self._ten_thousands_breaks = None
        self._thousands_breaks = None
        self._capital_cn_decimal = None
        self._capital_cn_integer = None
        self._capital_cn = None
        self._capital_cn_currency = None
        self._capital_cn_cny = None
        self._capital_cn_usd = None
        self._capital_cn_eur = None
        self._capital_en_decimal = None
        self._capital_en_integer = None
        self._capital_en = None
        self._capital_en_currency_decimal = None
        self._capital_en_currency_integer = None
        self._capital_en_currency = None
        self._capital_en_cny = None
        self._capital_en_usd = None
        self._capital_en_eur = None

    def __str__(self):
        return super().__str__()

    def is_integer(self):
        return isinstance(self, int)

    def is_float(self):
        return isinstance(self, int)

    def to_integer(self):
        return int(self)

    def to_float(self):
        return float(self)

    @property
    def string(self):
        return str(self)

    def is_negative(self):
        return self < 0

    def is_positive(self):
        return self > 0

    @property
    def decimal_string(self):
        if self._decimal_string is None:
            match = re.search(r'\.(\d+)', self.string)
            if match:
                self._decimal_string = match.group(1)
            else:
                self._decimal_string = '0'
        return self._decimal_string

    @property
    def integer_string(self):
        if self._integer_string is None:
            self._integer_string = str(self.to_integer())
        return self._integer_string

    @property
    def ten_thousands_breaks(self):
        if self._ten_thousands_breaks is None:
            self._ten_thousands_breaks = []
            for i in range(len(self.integer_string), 0, -4):
                self._ten_thousands_breaks.append(self.integer_string[max(i - 4, 0):i])
            # 分组正向
            self._ten_thousands_breaks.reverse()
            # 第一组不足4位时在前面用0补齐
            self._ten_thousands_breaks[0] = f"{self._ten_thousands_breaks[0]:0>4}"
        return self._ten_thousands_breaks

    @property
    def thousands_breaks(self):
        if self._thousands_breaks is None:
            self._thousands_breaks = []
            for i in range(len(self.integer_string), 0, -3):
                self._thousands_breaks.append(self.integer_string[max(i - 3, 0):i])
            # 分组正向
            self._thousands_breaks.reverse()
            # 第一组不足3位时在前面用0补齐
            self._thousands_breaks[0] = f"{self._thousands_breaks[0]:0>3}"
        return self._thousands_breaks

    @property  # 数字小数中文大写
    def capital_cn_decimal(self):
        if self._capital_cn_decimal is None:
            self._capital_cn_decimal = ''
            for i in range(0, len(self.decimal_string)):
                self._capital_cn_decimal += CN_INTEGER_0_TO_9[int(self.decimal_string[i])]
            if self._capital_cn_decimal == CN_INTEGER_0_TO_9[0]:
                self._capital_cn_decimal = ''
        return self._capital_cn_decimal

    def _capital_cn_by_ten_thousands_breaks_number(self, group_serial, number_serial):
        # 从万分位分组中取数字转为中文大写
        capital = f'{CN_INTEGER_0_TO_9[int(self.ten_thousands_breaks[group_serial][number_serial])]}'
        if self.ten_thousands_breaks[group_serial][number_serial] != '0' and number_serial < 3:
            capital += f'{CN_DECIMAL_UNITS[2 - number_serial]}'
        return capital

    @property  # 数字整数中文大写
    def capital_cn_integer(self):
        if self._capital_cn_integer is None:
            # 设定前面判断字符是不是非0
            first_zero = True
            last_zero = False
            self._capital_cn_integer = ''
            groups_quantity = len(self.ten_thousands_breaks)
            for group_serial in range(0, groups_quantity):
                capitals = ''
                group = self.ten_thousands_breaks[group_serial]
                for number_serial in range(4):
                    if group[number_serial] != '0':
                        if last_zero and not first_zero:
                            capitals += CN_INTEGER_0_TO_9[0]
                        capitals += self._capital_cn_by_ten_thousands_breaks_number(group_serial, number_serial)
                        first_zero = False
                        last_zero = False
                    else:
                        last_zero = True
                self._capital_cn_integer += capitals
                if group_serial < groups_quantity - 1:
                    self._capital_cn_integer += \
                        f'{CN_DECIMAL_4S_UNITS[groups_quantity - group_serial - 2]}'
            if not self._capital_cn_integer:
                self._capital_cn_integer = CN_INTEGER_0_TO_9[0]
        return self._capital_cn_integer

    @property  # 数字中文大写
    def capital_cn(self):
        if self._capital_cn is None:
            self._capital_cn = self.capital_cn_integer
            if self.capital_cn_decimal != '':
                self._capital_cn += CN_POINT_NAME + self.capital_cn_decimal
            if self.is_negative():
                self._capital_cn = f'{CN_MINUS} ' + self._capital_cn
        return self._capital_cn

    @property  # 货币中文大写
    def capital_cn_currency(self):
        if self._capital_cn_currency is None:
            self._capital_cn_currency = self.capital_cn_integer + CN_CURRENCY_UNIT
            decimal_cn = ''
            if len(self.decimal_string) > 0:
                if len(self.decimal_string) > 2 and self.decimal_string[2] != '0':
                    decimal_cn = CN_INTEGER_0_TO_9[int(self.decimal_string[2])] + CN_POINTS_UNITS[2]
                if len(self.decimal_string) > 1 and not (self.decimal_string[1] == '0' and decimal_cn == ''):
                    decimal_cn = CN_INTEGER_0_TO_9[int(self.decimal_string[1])] + CN_POINTS_UNITS[1] + decimal_cn
                if not (self.decimal_string[0] == '0' and decimal_cn == ''):
                    decimal_cn = CN_INTEGER_0_TO_9[int(self.decimal_string[0])] + CN_POINTS_UNITS[0] + decimal_cn
                self._capital_cn_currency += decimal_cn
            if not decimal_cn:
                self._capital_cn_currency += CN_ONLY_NAME
        return self._capital_cn_currency

    @property  # 数字中文大写
    def capital_cn_cny(self):
        if self._capital_cn_cny is None:
            self._capital_cn_cny = CN_CNY_NAME + self.capital_cn_currency
        return self._capital_cn_cny

    @property  # 数字中文大写
    def capital_cn_usd(self):
        if self._capital_cn_usd is None:
            self._capital_cn_usd = CN_USD_NAME + self.capital_cn_currency
        return self._capital_cn_usd

    @property  # 数字中文大写
    def capital_cn_eur(self):
        if self._capital_cn_eur is None:
            self._capital_cn_eur = CN_EUR_NAME + self.capital_cn_currency
        return self._capital_cn_eur

    @property  # 数字小数英文大写
    def capital_en_decimal(self):
        if self._capital_en_decimal is None:
            data = []
            for i in range(0, len(self.decimal_string)):
                data.append(EN_INTEGER_0_TO_9[int(self.decimal_string[i])])
            self._capital_en_decimal = ' '.join(data)
        return self._capital_en_decimal

    def _capital_en_by_thousands_breaks_add_and(self, group_serial, number_serial):
        g = self.thousands_breaks[group_serial]
        for i in range(number_serial + 1, 2):
            if g[i] != '0':
                return True
        if group_serial < len(self.thousands_breaks) - 1:
            for i in range(group_serial + 1, len(self._decimal_string)):
                for j in self.thousands_breaks[i]:
                    if j != 0:
                        return True
        return False

    def _capital_en_by_thousands_breaks(self, group_serial):
        g = self.thousands_breaks[group_serial]
        caps = []
        if g[0] != '0':
            caps.append(EN_INTEGER_0_TO_9[int(g[0])])
            caps.append(EN_HUNDRED_NAME)
            if self._capital_en_by_thousands_breaks_add_and(group_serial, 0):
                caps.append(EN_AND_NAME)
            caps.extend(self._capital_en_by_thousands_breaks_2(group_serial))
        else:
            caps.extend(self._capital_en_by_thousands_breaks_2(group_serial))
        return caps

    def _capital_en_by_thousands_breaks_2(self, group_serial):
        g = self.thousands_breaks[group_serial]
        caps = []
        if int(g[1]) >= 2:
            caps.append(EN_INTEGER_2NS[int(g[1]) - 2])
            caps.extend(self._capital_en_by_thousands_breaks_1(group_serial))
        elif g[1] == '1':
            caps.append(EN_INTEGER_1NS[int(g[2])])
        else:
            caps.extend(self._capital_en_by_thousands_breaks_1(group_serial))
        return caps

    def _capital_en_by_thousands_breaks_1(self, group_serial):
        g = self.thousands_breaks[group_serial]
        caps = []
        if g[2] != '0':
            caps.append(EN_INTEGER_0_TO_9[int(g[2])])
        return caps

    @property  # 数字整数英文大写
    def capital_en_integer(self):
        if self._capital_en_integer is None:
            groups_quantity = len(self.thousands_breaks)
            caps = []
            for i in range(0, len(self.thousands_breaks)):
                caps.extend(self._capital_en_by_thousands_breaks(i))
                if i < groups_quantity - 1:
                    caps.append(EN_DECIMAL_3S_UNITS[groups_quantity - i - 2])
            self._capital_en_integer = ' '.join(caps)
            if not self._capital_en_integer:
                self._capital_en_integer = EN_INTEGER_0_TO_9[0]
        return self._capital_en_integer

    @property  # 数字英文大写
    def capital_en(self):
        if self._capital_en is None:
            self._capital_en = self.capital_en_integer
            if self.capital_en_decimal != '' and self.capital_en_decimal != EN_INTEGER_0_TO_9[0]:
                self._capital_en += f' {EN_POINT_NAME} {self.capital_en_decimal}'
            if self.is_negative():
                self._capital_en = f'{EN_MINUS} ' + self._capital_en
        return self._capital_en

    @property  # 货币中文大写
    def capital_en_currency(self):
        if self._capital_en_currency is None:
            self._capital_en_currency = self.capital_en_integer
            ds = []
            if len(self.decimal_string) > 0:
                if len(self.decimal_string) > 2 and self.decimal_string[2] != '0':
                    ds.append(EN_POINTS_UNITS[2] if int(self.decimal_string[2]) in [0, 1] else f'{EN_POINTS_UNITS[2]}s')
                    ds.append(EN_INTEGER_0_TO_9[int(self.decimal_string[2])])
                if len(self.decimal_string) > 1 and not (self.decimal_string[1] == '0' and len(ds) <= 0):
                    ds.append(EN_POINTS_UNITS[1] if int(self.decimal_string[1]) in [0, 1] else f'{EN_POINTS_UNITS[1]}s')
                    ds.append(EN_INTEGER_0_TO_9[int(self.decimal_string[1])])
                if not (self.decimal_string[0] == '0' and len(ds) <= 0):
                    ds.append(EN_POINTS_UNITS[0] if int(self.decimal_string[0]) in [0, 1] else f'{EN_POINTS_UNITS[0]}s')
                    ds.append(EN_INTEGER_0_TO_9[int(self.decimal_string[0])])
                if len(ds) > 0:
                    ds.reverse()
                    self._capital_en_currency += f' {EN_AND_NAME} ' + ' '.join(ds)
                else:
                    self._capital_en_currency += f' {EN_ONLY_NAME}'
        return self._capital_en_currency

    @property
    def capital_en_cny(self):
        if self._capital_en_cny is None:
            self._capital_en_cny = f'{EN_CNY_NAME} {self.capital_en_currency}'
        return self._capital_en_cny

    @property
    def capital_en_usd(self):
        if self._capital_en_usd is None:
            self._capital_en_usd = f'{EN_USD_NAME} {self.capital_en_currency}'
        return self._capital_en_usd

    @property
    def capital_en_eur(self):
        if self._capital_en_eur is None:
            self._capital_en_eur = f'{EN_EUR_NAME} {self.capital_en_currency}'
        return self._capital_en_eur



from ...__templates__.styles import Styles


class Size(Styles):
    def __init__(self, senior):
        super().__init__(senior, [
            'pagination-lg',
            'pagination-sm',
        ])

    def set_large(self): return self.set('pagination-lg')

    def set_small(self): return self.set('pagination-sm')



from ...__templates__.styles import Styles


class Color(Styles):
    def __init__(self, senior):
        super().__init__(senior, [
            'btn-primary',
            'btn-secondary',
            'btn-success',
            'btn-danger',
            'btn-warning',
            'btn-info',
            'btn-light',
            'btn-dark',
        ])

    def set_primary(self): return self.set('btn-primary')

    def set_secondary(self): return self.set('btn-secondary')

    def set_success(self): return self.set('btn-success')

    def set_danger(self): return self.set('btn-danger')

    def set_warning(self): return self.set('btn-warning')

    def set_info(self): return self.set('btn-info')

    def set_light(self): return self.set('btn-light')

    def set_dark(self): return self.set('btn-dark')



from .core import String
import string
import secrets


class StringRandom(String):
    def __new__(cls, length: int = 16):
        if length <= 0:
            raise ValueError("随机字符串长度必须为正整数")
        alphabet = string.ascii_letters + string.digits
        return str.__new__(cls, ''.join(secrets.choice(alphabet) for _ in range(length)))



from ...nodes import Tag


class InputText(Tag):
    def __init__(self, senior, name: str = None, value: str | int | float = None, placeholder: str = None):
        super().__init__(senior, "input")
        self.output_break_inner = False
        self.value: str | int | float = "" if value is None else value
        if name is not None:
            self.attributes["name"] = name
        if placeholder is not None:
            self.attributes["placeholder"] = placeholder



from ...tags.button.core import Button as ButtonTag


class Button(ButtonTag):
    def __init__(self, senior, text=None):
        super().__init__(senior, text)
        self.attributes.classes.set("btn")
        self._color = None
        self._color_outline = None
        self._size = None

    @property
    def color(self):
        if self._color is None:
            from .color import Color
            self._color = Color(self)
        return self._color

    @property
    def color_outline(self):
        if self._color_outline is None:
            from .color_outline import Color
            self._color_outline = Color(self)
        return self._color_outline

    @property
    def size(self):
        if self._size is None:
            from .size import Size
            self._size = Size(self)
        return self._size



from ..digital.core import Digital


class String(str):
    def __new__(cls, characters):
        try:
            characters = str(characters)
        except (TypeError, ValueError) as e:
            raise ValueError(f"输入变量不能转化为字符串: {characters}") from e
        return str.__new__(cls, characters)

    def contains(self, part: str):
        if not isinstance(part, str):
            raise ValueError("检查部分必须是字符串")
        return part in self

    def is_digitalizable(self):
        try:
            float(self.strip())
            return True
        except ValueError:
            return False

    def digitize(self) -> Digital:
        if self == '':
            return Digital(0)
        if not self.is_digitalizable():
            raise ValueError(f"字符不能转化为数浮点数: {self}")
        if '.' in self.strip():
            return Digital(float(self.strip()))
        return Digital(int(self.strip()))



from ....__templates__.styles import Styles


class Color(Styles):
    def __init__(self, senior):
        super().__init__(senior, [
            'table-primary',
            'table-secondary',
            'table-success',
            'table-danger',
            'table-warning',
            'table-info',
            'table-light',
            'table-dark',
        ])

    def set_primary(self): return self.set('table-primary')

    def set_secondary(self): return self.set('table-secondary')

    def set_success(self): return self.set('table-success')

    def set_danger(self): return self.set('table-danger')

    def set_warning(self): return self.set('table-warning')

    def set_info(self): return self.set('table-info')

    def set_light(self): return self.set('table-light')

    def set_dark(self): return self.set('table-dark')



import math
import numpy


class Math:
    @staticmethod
    def x2y_by_polynomial_regression(x, x_list, y_list, degree: int = 2):
        """
        :param x: 获取y需要对应的x
        :param x_list: 基础x数组
        :param y_list: 基础y数组
        :param degree: 幂
        :return: 返回 x对应的y
        """
        cs = numpy.polyfit(x_list, y_list, degree)
        r = 0.0
        for i in range(degree, 0, -1):
            r += pow(x, i) * cs[degree - i]
        r += cs[-1]
        return r

    @staticmethod
    def ring_volume_weight(diameter: float, height: float, inner_diameter: float = 0, margin: float = 0,
                           density: float = 1):
        diameter += 2 * margin
        height += 2 * margin
        if inner_diameter < 2 * margin:
            inner_diameter = 0
        else:
            inner_diameter -= 2 * margin
        vol = math.pi * (pow(diameter / 2, 2) - pow(inner_diameter / 2, 2)) * height
        return vol * density

    @staticmethod
    def ring_min_diameter_by_pressure(inner_diameter: float = 0, weight: float = 1000000,
                                      safety_ratio: float = 5, yield_strength: float = 27.6):
        """
        :param inner_diameter: mm
        :param weight: kgs 默认按1000吨硫化机压力
        :param safety_ratio: num 安全系数 默认 5 倍
        :param yield_strength: kgs/mm^2 默认ZG 270-500 铸钢的 屈服强度 YIELD STRENGTH (/Mpa) 270 N/mm2 (27.6 kgf/mm2)
        :return: minimal_diameter: mm
        """
        area = weight / yield_strength * safety_ratio
        return pow(area / math.pi + pow(inner_diameter / 2, 2), 0.5) * 2


class Classes(list):
    def reset(self, classes: str | list = None) -> bool:
        self.clear()
        if classes is None:
            return True
        return self.set(classes)

    def set(self, classes: str | list = None) -> bool:
        if isinstance(classes, str):
            if ' ' in classes:
                self.set(classes.split(' '))
            else:
                if classes not in self:
                    self.append(classes)
            return True
        elif isinstance(classes, list):
            for c in classes:
                self.set(c)
            return True
        return False

    def unset(self, classes: str | list = None) -> bool:
        if isinstance(classes, str):
            if ' ' in classes:
                self.unset(classes.split(' '))
            else:
                if classes in self:
                    self.remove(classes)
            return True
        elif isinstance(classes, list):
            for c in classes:
                self.unset(c)
            return True
        return False

    @property
    def is_empty(self) -> bool: return len(self) <= 0

    def __str__(self): return ' '.join(self)

    def output(self) -> str: return self.__str__()



import sys
from colorama import Style


class Terminal:
    def __init__(self):
        self._color = ''
        self._background = ''

    def set_color(self, color: str):
        self._color = color

    def set_background(self, color: str):
        self._background = color

    def input(self, content: str):
        try:
            inputs = input(self._background + self._color + content + Style.RESET_ALL)
            sys.stdout.flush()
            return inputs
        except UnicodeDecodeError as e:
            print(f"编码错误：{e}")

    def print(self, content: str, line_break=True):
        print(self._background + self._color + content + Style.RESET_ALL, end='\n' if line_break else '')
        sys.stdout.flush()

    @property
    def color(self):
        from .color_fore import ColorFore
        return ColorFore(self, True)

    @property
    def background(self):
        from .color_back import ColorBack
        return ColorBack(self)



import inspect
import os.path
import subprocess
import shutil


class Pypi:
    def __init__(self, module, python_exe=None):
        self._module = module
        self._name = None
        self._version = None
        self._description = None
        self._long_description = None
        self._long_description_file = None
        self._long_description_content_type = None
        self._packages = []
        self._install_requires = {}
        self._entry_point_console_scripts = {}
        self._python_requires = '3.12'
        self._output_build_base = ''
        self._output_dist_dir = ''
        self._output_egg_base = ''
        self._setup_py = 'pack_upload_setup.py'
        self._python_exe = python_exe

    @property
    def module(self):
        return self._module

    @property
    def name(self):
        if self._name is None:
            if hasattr(self.module, 'NAME'):
                return getattr(self.module, 'NAME')
            return None
        return self._name

    @name.setter
    def name(self, name: str):
        self._name = name

    @property
    def version(self):
        if self._version is None:
            if hasattr(self.module, 'VERSION'):
                return getattr(self.module, 'VERSION')
            return None
        return self._version

    @version.setter
    def version(self, version: str):
        self._version = version

    @property
    def description(self):
        if self._description is None:
            if hasattr(self.module, 'DESCRIPTION'):
                return getattr(self.module, 'DESCRIPTION')
            return None
        return self._description

    @description.setter
    def description(self, description: str):
        self._description = description

    @property
    def packages(self):
        return self._packages

    def add_package(self, name: str):
        if name not in self._packages:
            self._packages.append(name)

    @property
    def long_description(self):
        if self._long_description is None:
            if hasattr(self.module, 'LONG_DESCRIPTION'):
                return getattr(self.module, 'LONG_DESCRIPTION')
            return None
        return self._long_description

    @long_description.setter
    def long_description(self, file_name: str):
        self._long_description = file_name

    @property
    def long_description_file_path(self):
        if self._long_description_file is not None:
            return self._long_description_file
        if self.long_description is not None:
            return 'README.MD'
        return None

    @property
    def long_description_content_type(self):
        if self._long_description_content_type is None:
            self.set_long_description_content_type_text()
        return self._long_description_content_type

    def set_long_description_content_type_text(self):
        self._long_description_content_type = 'text/plain'

    def set_long_description_content_type_markdown(self):
        self._long_description_content_type = 'text/markdown'

    @property
    def install_requires(self):
        return self._install_requires

    def add_install_require(self, name: str, version: str):
        self._install_requires[name] = version

    @property
    def entry_point_console_scripts(self):
        return self._entry_point_console_scripts

    def add_entry_point_console_script(self, name: str, method: str):
        self._entry_point_console_scripts[name] = method

    @property
    def python_requires(self):
        return self._python_requires

    @python_requires.setter
    def python_requires(self, version: str):
        self._python_requires = version

    @property
    def output_build_base(self):
        return self._output_build_base

    @output_build_base.setter
    def output_build_base(self, path: str):
        self._output_build_base = path

    @property
    def output_dist_dir(self):
        return self._output_dist_dir

    @output_dist_dir.setter
    def output_dist_dir(self, path: str):
        self._output_dist_dir = path

    @property
    def output_egg_base(self):
        return self._output_egg_base

    @output_egg_base.setter
    def output_egg_base(self, path: str):
        self._output_egg_base = path

    @property
    def setup_py(self):
        return self._setup_py

    @setup_py.setter
    def setup_py(self, filename: str):
        self._setup_py = filename

    @property
    def python_exe(self):
        return self._python_exe

    @python_exe.setter
    def python_exe(self, python_exe: str):
        self._python_exe = python_exe

    def clear(self):
        # 删除目录及其内容
        try:
            # if os.path.isdir('build'):
            shutil.rmtree('build')
            # if os.path.isdir('dist'):
            shutil.rmtree('dist')
            # if os.path.isdir(f'{self.name}.egg-info'):
            shutil.rmtree(f'{self.name}.egg-info')
            # if os.path.isfile(self.setup_py):
            os.remove(self.setup_py)
        except Exception:
            pass

    def build(self):
        self.clear()
        lines = [
            '# -*- coding: utf-8 -*-',
            'from setuptools import setup, find_packages',
            'setup(',
            f'    name=\'{self.name if self.name else 'My Module'}\',',
            f'    version=\'{self.version if self.version else '1.0.0'}\',',
        ]
        if self.description:
            lines.append(f'{' ' * 4}description=\'{self.description}\',')
        if self.long_description:
            lines.append(f'{' ' * 4}long_description=open(r\'{self.long_description_file_path}\','
                         f' encoding=\'utf-8\').read(),')
            lines.append(f'{' ' * 4}long_description_content_type=\'{self.long_description_content_type}\',')
        if len(self.packages) > 0:
            lines.append(f'{' ' * 4}packages=find_packages(include=[')
            for n in self.packages:
                lines.append(f'{' ' * 8}\'{n}\',')
                lines.append(f'{' ' * 8}\'{n}.*\',')
            lines.append(f'{' ' * 4}]),')
        else:  # find_packages
            lines.append(f'{' ' * 4}packages=[')
            lines.append(f'{' ' * 4}packages=find_packages(),')
            lines.append(f'{' ' * 4}],')

        # 打包所有内部markdown文件
        lines.append(f'{' ' * 4}package_data={{')
        if len(self.packages) > 0:
            for n in self.packages:
                lines.append(f'{' ' * 8}\'{n}\': [\'*.md\', \'*.MD\'],')
        else:
            lines.append(f'{' ' * 8}\'\': [\'*.md\', \'*.MD\'],')
        lines.append(f'{' ' * 4}}},')

        lines.append(f'{' ' * 4}install_requires=[')
        if len(self.install_requires) > 0:
            for n, v in self.install_requires.items():
                lines.append(f'{' ' * 8}\'{n}=={v}\',')
        else:
            with open('requirements.txt', 'r') as f:
                # 逐行读取文件内容并存储到数组中
                ls = f.readlines()
            # for i in hebill.file.File('requirements.txt').read_lines():
            for i in ls:
                i = i.strip()
                if '~=' in i:
                    n, v = i.split('~=')
                    if n == 'setuptools':
                        continue
                    lines.append(f'{' ' * 8}\'{n}=={v}\',')
        lines.append(f'{' ' * 4}],')
        if len(self.entry_point_console_scripts) > 0:
            lines.append(f'{' ' * 4}entry_points={{')
            lines.append(f'{' ' * 8}\'console_scripts\':[')
            for n, v in self.entry_point_console_scripts.items():
                lines.append(f'{' ' * 12}\'{n} = {v}\':[')
            lines.append(f'{' ' * 8}],,')
            lines.append(f'{' ' * 4}}},')
        lines.append(f'{' ' * 4}python_requires=\'>={self.python_requires if self.python_requires else '3.12'}\',')
        lines.append(')\n')
        try:
            with open(self.setup_py, 'w', encoding='utf-8') as file:
                file.write('\n'.join(lines))
            print(f'{self.setup_py} 文件已经生成：{self.setup_py}')
        except (Exception, IOError) as e:
            print(f'{self.setup_py} 文件生成失败[{e}]')
            return

    def pack(self, upload=True, clear=True):
        self.build()
        if self.python_exe is None:
            print('请设置python.exe路径')
            return
        command = [self.python_exe, self.setup_py, 'sdist', 'bdist_wheel']

        print(f"执行命令：{' '.join(command)}")
        result = subprocess.run(command)

        if result.returncode == 0:
            print("打包处理执行完成，下面开始上传")
        else:
            print("打包处理执行失败！")
            return
        if not upload:
            return
        command = ['twine', 'upload', 'dist/*']
        pypirc = os.path.join(os.path.expanduser("~"), '.pypirc')
        # pypirc = hebill.dir.Dir(os.path.expanduser("~")).sub_file('.pypirc')
        if not os.path.exists(pypirc):
            print(f'没有查询到文件{pypirc}, 请手动上传：{' '.join(command)}')
        else:
            print(f"已找到认证文件{pypirc}，执行命令：{' '.join(command)}")
            result = subprocess.run(command)
            if result.returncode == 0:
                print("上传成功")
                if clear:
                    self.clear()
            else:
                print(f"上传失败！请检查错误信息，或手动上传：{' '.join(command)}")
                return
